'' =================================================================================================
''
''   File....... jm_i2c_scan.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2014 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 09 FEB 2014
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq               ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { io pins }

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30
  
  SDA = 29                                                      ' eeprom / i2c
  SCL = 28

  MPUsda  = 8       ' All I2C Sensors on SAME Bus
  MPUscl  = 9

  BMPSCL   = 12   
  BMPSDA   = 13

  I2C_SDA = 10
  I2C_SCL = 11  

con

  #0, LSBFIRST, MSBFIRST
  

con { pst formatting }

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR
  #14, GOTOX, GOTOY, CLS


  bmpAddr = $EE
  mpuAddr = $D0

obj

  term : "fullduplexserial"
  i2c  : "jm_i2c_Mod"
 
  MPU           : "MPU-6050"               ' 1 Cog      *** MPU Loop
  'MPU           : "MPU-9150"               ' 1 Cog
  f             : "FloatMath"              ' 0 Cog
  fS            : "FloatString"            ' 0 Cog
  umath :       "umath"                                 '0 Cog
  
  
var

 ' MPU-9150 $D0 %1101_0000
 ' BMP-180  $EE %1110_1110  

  long ac1, ac2, ac3, ac4, ac5, ac6, b1, b2, mb, mc, md, id
  long UT, UP, oss

 

pub main | addr, check

  term.start(RX1, TX1, %0000, 115_200)
  i2c.setupx(I2C_SCL, I2C_SDA)

  'MPU.Startx(I2C_SCL, I2C_SDA) 
  
  ' open PST, enable, then press a key
  'repeat
  '  Read_MPU

  oss := 1
  
  read_BMP
  
  repeat
    term.rxflush
    term.rx
     
    term.tx(CLS)
    term.str(string("I2C Scanner", CR, CR))
     
    repeat addr from %0000_0010 to %1111_1110 step %10            ' ignore r/w bit
      ihex2(addr)
      repeat 3
        term.tx(" ")
      ibin8(addr)
      repeat 3
        term.tx(" ")
        
      i2c.start
      check := i2c.write(addr)
      i2c.stop
      if (check == i2c#ACK)
        term.str(string("Yes", CR))
      else
        term.str(string("No", CR))


      if addr == $EE            ' BMP Found
        Read_BMP

      if addr == $D0            ' MPU Found
        Read_MPU
                      
      pause(50)

     
Pri Read_MPU | X, Y, Z, A, B, C    

    ' // Get New Accleerometer Values
    X := f.fFloat(MPU.GetAX)
    Y := f.fFloat(MPU.GetAY)
    Z := f.fFloat(MPU.GetAZ)
    
    ' // Get New Gyro Values
    A := f.fFloat(MPU.GetRX)
    B := f.fFloat(MPU.GetRY)
    C := f.fFloat(MPU.GetRZ)

    term.str(fs.floattostring(X))
    term.str(string(","))
    term.str(fs.floattostring(Y))
    term.str(string(","))
    term.str(fs.floattostring(Z))
    term.str(string(","))        
    term.str(fs.floattostring(A))
    term.str(string(","))
    term.str(fs.floattostring(B))
    term.str(string(","))
    term.str(fs.floattostring(C))        
    term.tx(13)


    
Pri Read_BMP | pTmp, tTmp
    
      {
      Table 5: Calibration Coefficients
                  BMP180 reg adr
      Parameter   MSB        LSB
        AC1       0xAA       0xAB
        AC2       0xAC       0xAD
        AC3       0xAE       0xAF
        AC4       0xB0       0xB1
        AC5       0xB2       0xB3
        AC6       0xB4       0xB5
        B1        0xB6       0xB7
        B2        0xB8       0xB9
        MB        0xBA       0xBB
        MC        0xBC       0xBD
        MD        0xBE       0xBF
      }

    ' i2c.readLocation(SCL,device_address, register)

  repeat
      
    ' // Read Calibration Coefficients
    ac1 := i2c.readLocation(bmpAddr, $AA) << 8 
    ac1 |= i2c.readLocation(bmpAddr, $AB)
     
    'ac1 := i2c.readLocation(bmpAddr, $AA)  <<  24
    'ac1  ~>= 16   ' shift arith right
    'ac1 |= i2c.readLocation(bmpAddr, $AB)   
     
    ac2 := i2c.readLocation(bmpAddr, $AC) << 8 
    ac2 |= i2c.readLocation(bmpAddr, $AD)  
    ac3 := i2c.readLocation(bmpAddr, $AE) << 8 
    ac3 |= i2c.readLocation(bmpAddr, $AF) 
     
    ac4 := i2c.readLocation(bmpAddr, $B0) << 8 
    ac4 |= i2c.readLocation(bmpAddr, $B1) 
     
    ac5 := i2c.readLocation(bmpAddr, $B2) << 8 
    ac5 |= i2c.readLocation(bmpAddr, $B3)  
    ac6 := i2c.readLocation(bmpAddr, $B4) << 8 
    ac6 |= i2c.readLocation(bmpAddr, $B5) 
           
    b1 := i2c.readLocation(bmpAddr, $B6) << 8   
    b1 |= i2c.readLocation(bmpAddr, $B7) 
    b2 := i2c.readLocation(bmpAddr, $B8) << 8   
    b2 |= i2c.readLocation(bmpAddr, $B9) 
           
    mb := i2c.readLocation(bmpAddr, $BA) << 8  
    mb |= i2c.readLocation(bmpAddr, $BB) 
    mc := i2c.readLocation(bmpAddr, $BC) << 8  
    mc |= i2c.readLocation(bmpAddr, $BD)    
    md := i2c.readLocation(bmpAddr, $BE) << 8  
    md |= i2c.readLocation(bmpAddr, $BF) 
     
    id := i2c.readLocation(bmpAddr, $D0)   ' 55     *** Should = 0x55
     
    waitcnt((clkfreq / 4) + cnt)

    term.str(string("AC1: "))
    term.hex(ac1, 2)
    term.str(string("  "))
    term.dec(ac1)
    term.tx(13)
    term.str(string("AC2: "))
    term.hex(ac2, 2)
    term.str(string("  "))
    term.dec(ac2)    
    term.tx(13)
    term.str(string("AC3: "))
    term.hex(ac3, 2)
    term.str(string("  "))
    term.dec(ac3)    
    term.tx(13)
    term.str(string("AC4: "))
    term.hex(ac4, 2)
    term.str(string("  "))
    term.dec(ac4)    
    term.tx(13)
    term.str(string("AC5: "))
    term.hex(ac5, 2)
    term.str(string("  "))
    term.dec(ac5)    
    term.tx(13)
    term.str(string("AC6: "))
    term.hex(ac6, 2)
    term.str(string("  "))
    term.dec(ac6)    
    term.tx(13)                    

    term.str(string("B1: "))
    term.hex(b1, 2)
    term.str(string("  "))
    term.dec(b1)    
    term.tx(13)
    term.str(string("B2: "))
    term.hex(b2, 2)
    term.str(string("  "))
    term.dec(b2)    
    term.tx(13)

    term.str(string("MB: "))
    term.hex(mb, 2)
    term.str(string("  "))
    term.dec(mb)    
    term.tx(13)
    term.str(string("MC: "))
    term.hex(mc, 2)
    term.str(string("  "))
    term.dec(mc)    
    term.tx(13)
    term.str(string("MD: "))
    term.hex(md, 2)
    term.str(string("  "))
    term.dec(md)
    term.tx(13)        

    term.str(string("ID: "))
    term.hex(id, 2)
    term.str(string("  "))
    term.dec(id)    
    term.tx(13)
    
    ' // Read Uncompressed Temperature Value
    ' Write 0x2E into reg 0xF4, wait 4.5ms
    ' Read reg 0xF6 (MSB), 0xF7 (LSB)

    i2c.writeLocation(bmpAddr, $F4, $2E)       ' Temperature  0x2E  4.5
    waitcnt((clkfreq / 222) + cnt)             ' 4.5ms
    UT := i2c.readLocation(bmpAddr, $F6) << 8
    UT |= i2c.readLocation(bmpAddr, $F7) 

    term.str(string("UT: "))
    term.hex(UT, 6)
    term.str(string("   "))
    term.dec(UT)
    term.tx(13)
    
    ' // Read Uncompressed Pressure Value
    ' Write 0x34+(oss<<6) into reg 0xF4, wait #.#ms
    ' Read reg 0xF6 (MSB), 0xF7 (LSB), 0xF8 (XLSB)

    'i2c.writeLocation(bmpAddr, $F4, $34 + (oss<<6))

    ' i2c.readLocation(SCL,device_address, register)
    ' i2c.writeLocation(SCL,device_address, register, value) 
    
    if oss == 0
      i2c.writeLocation(bmpAddr, $F4, $34)     ' Pressure (oss = 0)  0x34  4.5
      waitcnt((clkfreq / 222) + cnt)                         ' 4.5ms    
    elseif oss == 1
      i2c.writeLocation(bmpAddr, $F4, $74)     ' Pressure (oss = 1)  0x74  7.5
      waitcnt((clkfreq / 133) + cnt)                         ' 7.5ms    
    elseif oss == 2
      i2c.writeLocation(bmpAddr, $F4, $B4)     ' Pressure (oss = 2)  0xB4  13.5
      waitcnt((clkfreq / 74) + cnt)                          ' 13.5ms   
    elseif oss == 3
      i2c.writeLocation(bmpAddr, $F4, $F4)     ' Pressure (oss = 3)  0xF4  25.5
      waitcnt((clkfreq / 39) + cnt)                          ' 25.5ms

    UP := i2c.readLocation(bmpAddr, $F6) << 8
    UP |= i2c.readLocation(bmpAddr, $F7) << 8
    UP |= i2c.readLocation(bmpAddr, $F8) 
      
    term.str(string("UP: "))
    term.hex(UP, 6)
    term.str(string("   "))
    term.dec(UP)
    term.tx(13)        


    ' // Calculate True Temperature

    ' // Calculate True Pressure

    Convert(oss, tTmp, pTmp)

    term.str(string("Temperature: "))  
    term.dec(tTmp)
    term.tx(13)
    term.str(string("Pressure: "))
    term.dec(pTmp)
    term.tx(13)
    term.tx(13)     

    
    'waitcnt(clkfreq + cnt)


PRI Convert(mode, TempPtr, PressurePtr) | x1, x2, b5, b6, x3, b3, p, b4, th
''
  x1 := ((ut - ac6) * ac5) ~> 15
  x2 := (mc << 11) / (x1 + md)
  b5 := x1 + x2
  long[TempPtr] := (b5 + 8) ~> 4
        
  b6 := b5 - 4000
  x1 := (b2 * ((b6 * b6) ~> 12)) ~> 11
  x2 := (ac2 * b6) ~> 11
  x3 := x1 + x2
  b3 := ((((ac1 << 2) + x3) << mode) + 2) ~> 2

  x1 := (ac3 * b6) ~> 13
  x2 := (b1 * ((b6 * b6) ~> 12)) ~> 16
  x3 := ((x1 + x2) + 2) ~> 2

  'b4 := (ac4 * (x3 + 32768)) >> 15                                             'unsigned 32 bit multiple
  b4 := umath.multdiv(ac4, (x3 + 32768), 32768)

  'b7 := (up - b3) * (50000 >> mode)                                            'unsigned 32 bit multiple
  'if b7 & $80000000
  '  p := (b7 / b4) << 1
  'else
  '  p := (b7 * 2) / b4
  p := umath.multdiv((up - b3), (100000 >> mode), b4)

  th := p ~> 8 
  x1 := th * th
  x1 := (x1 * 3038) ~> 16
  x2 := (-7357 * p) ~> 16
  long[PressurePtr] := p + ((x1 + x2 + 3791) ~> 4)

con

  { ------------- }
  {  B A S I C S  }
  { ------------- }


pub ihex2(value)

  term.tx("$")
  term.hex(value, 2)


pub ibin8(value)

  term.tx("%")
  term.bin(value >> 4, 4)
  term.tx("_")
  term.bin(value & $0F, 4)
  

con

  { ------------- }
  {  B A S I C S  }
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]


dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}